#***************************************************************************
#*          Copyright 2000 DBAmon Software, All Rights Reserved            *
#***************************************************************************
# -----------------------------------------------------------------------------
function disk_bal()
# -----------------------------------------------------------------------------

{
  # 
  # Cleanup (in case it didn't happen last time)
  #

  function_cleanup()

  # 
  # First, run onstat -D to get the largest read value (for graph scale)
  #

  system("onstat -D | grep \" /\" | sort -bnr +4 > "tmpfl2)
  
  raw = "===================================="
  getline < tmpfl2
 

  scale = ($5 / length(raw)) * 2
  close tmpfl2 

  if (scale == 0)
  {
    printf("db_spaces | Error: scale value == 0\n")
    system("cat "tmpfl2)
    return 1
  }

  # 
  # Gather read data
  #

  system("onstat -D | grep \" /\" | sort -b +6 > "tmpfl2)

  flag = 0
  stop = 0

  msg = sprintf("Disk I/O Distribution Usage Report\n")
  print msg 

  printf(" Disk Path                                    Reads    Writes     Total\n") 
  printf(" ---------------------------------------- --------- --------- ---------\n") 

  last_disk = ""

  t_reads  = 0
  t_writes = 0
  t_total  = 0

  while (getline < tmpfl2 > 0 && stop == 0)
  {
    if (NF != 7)
    {
      printf("db_spaces | Error: onstat -D output doesn't contain 7 fields\n")
      system("cat "tmpfl2)
      return 1
    }

    if ($7 != last_disk)
    {
      if (last_disk != "")
      {
        d_graph = substr(raw,1,d_total / scale) ">"
        printf(" %-40s %9i %9i %9i %s\n",last_disk,d_reads,d_writes,d_total,d_graph) 
      }

      last_disk = $7
      d_reads   = $5
      d_writes  = $6
      d_total   = d_reads + d_writes

    }
    else
    {
      d_reads   += $5
      d_writes  += $6
      d_total   = d_reads + d_writes
    } 

    t_reads  += $5
    t_writes += $6
    t_total   = t_reads + t_writes

  }
  close tmpfl2 

  if (scale == 0) scale = d_total

  d_graph = substr(raw,1,d_total / scale) ">"
  printf(" %-40s %9i %9i %9i %s\n",$7,d_reads,d_writes,d_total,d_graph) 

  last_disk = "" 
  printf("                                          ========= ========= =========\n") 
  printf(" %-40s %9i %9i %9i\n",last_disk,t_reads,t_writes,t_total) 
  printf("\n") 
  
  #system("cat "tmpfl2)
  #printf("db_spaces | disk_bal Done\n")

}


# -----------------------------------------------------------------------------
function stat_d()
# -----------------------------------------------------------------------------

{

  function_cleanup()

  if (system("onstat -d > "tmpfl2) != 0)
  {
    printf("db_spaces | onstat error\n")
    exit 1
  }
  
  flag = 0
  stop = 0

  max_dbsp = 0
  num_lines = 0

  #
  # Delete previous variables
  #

  for (x in dbsp) delete dbsp[x]
  for (x in dbspsize) delete dbspsize[x]
  for (x in dbspfree) delete dbspfree[x]
  for (x in dbspused) delete dbspused[x]
  for (x in dbspchks) delete dbspchks[x]
  for (x in chunk) delete chuck[x]  
  for (x in dbsnum) delete dbsnum[x]
  for (x in offset) delete offset[x] 
  for (x in path) delete path[x]  
  for (x in size) delete size[x]  
  for (x in free) delete free[x] 
  for (x in thru) delete thru[x]
  for (x in dbspsize) delete dbspsize[x] 
  for (x in dbspfree) delete dbspfree[x] 
  for (x in dbspchks) delete dbspchks[x] 
  for (x in dbspused) delete dbspused[x]
  for (x in dbsppctu) delete dbsppctu[x] 
  for (x in pctused) delete pctused[x]

  #
  # get list of dbspaces
  #

  while (getline < tmpfl2 > 0 && stop == 0)
  {
    if ($2 == "active,") 
    {
      stop = 1 
      flag = 0 
    }

    if (flag == 1)
    {
      dbsp[$2] = $NF
    
      dbspsize[$2] = 0
      dbspfree[$2] = 0
      dbspused[$2] = 0
      dbspchks[$2] = 0
	
      if ($2 > max_dbsp) max_dbsp = $2
    }


    if ($1 == "address") flag = 1 
  }
   
  close tmpfl2

  if (debug) printf("debug: max_dbsp=%s\n",max_dbsp) 

  #
  # Get disk space utilization data
  #

  system("onstat -d|egrep 'PO-|MO-'  > "tmpfl1)

  i=0
  flag=0

  while (getline < tmpfl1 > 0)
  {
    if (NF == 8) 
    {

      chunk[i]  = $2     
      dbsnum[i] = $3     
      offset[i] = $4     
      path[i]   = $8
      size[i]   = $5
      free[i]   = $6
      flags[i]  = substr($7,1,1)
      thru[i]   = offset[i] + size[i] - 1

      if (flags[i] == "P")
      {
	dbspsize[$3] += $5
	dbspfree[$3] += $6
	dbspchks[$3] ++ 
      }

      if (debug) print "dbsp=" dbsp[$3] " " dbspsize[$3] " " dbspfree[$3] " " dbspchks[$3]

      if (size[i] == 0)
      {
        printf("db_spaces | zero divide #4\n")
        printf("db_spaces | dbsp=\n",dbsp[i])
        system("cat "tmpfl1)
        msg = sprintf("db_spaces | zero divide #2\n")
        printf(msg)
        return 1
      }

      pctused[i] = (size[i]-free[i])/size[i]*100

      gb_warn = 0

      i++

    }

  }

  close tmpfl1

  num_lines = i-1

  for (i=0; i<= max_dbsp; i++)
  {

    if (dbsp[i] == "") continue

    if (dbspsize[i] == 0)
    {
      printf("dbamon | "title[thishost]" | zero divide #6\n")
      dbspsize[i] = 1
    }

    dbspused[i] = (dbspsize[i]-dbspfree[i])
    dbsppctu[i] = (dbspsize[i]-dbspfree[i])/dbspsize[i]*100
  }

  #
  # Create CHUNK Disk Space Report 
  #

  if (level > 1)
  {
    msg = sprintf("Disk Usage Report\n") 
    print msg 

    msg = sprintf("Chunk Report (Units: MB - Sorted by Device)\n") 
    print msg

    msg = sprintf(" DBSpace           P/M Chunk Unix Device                               Offset     End    Size    Free    Used Pct Full")
    print msg 
    msg = sprintf(" ----------------- --- ----- --------------------------------------- -------- ------- ------- ------- ------- --------")
    print msg

    total_free = 0
    total_size = 0
    total_used = 0

    system("rm " tmpflt " > /dev/null 2>&1")

    for (i=0; i<= num_lines; i++)
    {
      used[i] = size[i] - free[i]

      msg = sprintf(" %-17s %-3s %5s %-40s %7d %7d %7d %7d %7d   %5.0f%%",dbsp[dbsnum[i]],flags[i],chunk[i],path[i],offset[i]/500,thru[i]/500,size[i]/500,free[i]/500,used[i]/500,pctused[i])
      print msg >> tmpflt

      total_free += free[i] 
      total_size += size[i] 
      total_used += used[i]  
    }    

    if (total_size == 0)
    {
      printf("db_spaces | zero divide #1\n")
      return 1
    }

    close tmpflt
    system("sort +3 +4 " tmpflt)

    msg = sprintf("                                                                                      ======= ======= ======= ========")
    print msg 

    msg = sprintf("                                                                                      %7d %7d %7d   %5.0f%%",total_size/500,total_free/500,total_used/500,total_used/total_size*100)
    print msg 
  }
  
#  
# Create DBSPACE Report
#

  msg = sprintf("\nDBSpace Report (Units: MB)\n")
  print msg 

  msg = sprintf(" DBSpace           Chunks       Size       Free       Used   Pct Full")
  print msg 
  msg = sprintf(" ----------------- ------ ---------- ---------- ---------- ----------")
  print msg 

  total_size = 0
  total_free = 0
  total_used = 0
  total_chks = 0

  system("rm " tmpflt " > /dev/null 2>&1")

  for (i=0; i<= max_dbsp; i++)
  {

    if (dbsp[i] != "")
    {
      #if (dbspsize[i]/500 < 1) dbspsize[i] = 500
      #if (dbspfree[i]/500 < 1) dbspfree[i] = 500
      #if (dbspused[i]/500 < 1) dbspused[i] = 500

      msg = sprintf(" %-20s  %2s  %9.1d  %9.1d  %9.1d    %6.0f%%",dbsp[i],dbspchks[i],dbspsize[i]/500,dbspfree[i]/500,dbspused[i]/500,dbsppctu[i])
      print msg >> tmpflt

      total_size += dbspsize[i]/500
      total_free += dbspfree[i]/500
      total_used += dbspused[i]/500
      total_chks += dbspchks[i]
    }

  }

  close tmpflt
  system("sort " tmpflt)

  msg = sprintf("                   ====== ========== ========== ========== ==========")
  print msg 

  if (total_size == 0)
  {
    printf("db_spaces | zero divide #5\n")
    return 1
  }

  msg = sprintf("               %10d %10d %10d %10d %9.0f%% \n",total_chks,total_size,total_free,total_used,total_used/total_size*100)
  print msg 

  grand_total_size += total_size / 1000

#  
# Create Disk I/O Balance Report
#

  if (level > 2) 
  {
    disk_bal()
  }

}
#  
# Clean up
#

function_cleanup()
{
  close report_path 
}


# -----------------------------------------------------------------------------
function function_cleanup()
# -----------------------------------------------------------------------------
{
  system("rm /tmp/db_spaces_tmp* > /dev/null 2>&1")
}


# -----------------------------------------------------------------------
# MAIN Logic
# -----------------------------------------------------------------------
{

  #
  # debug flags
  #

  debug      = 0
  debug_html = 0
  debug_read = 0
  debug_opc  = 0
  debug_crit = 0

  #
  # set other variables
  #

  # Cleanup old temp files
  system("rm /tmp/db_spaces* > /dev/null 2>&1")

  tmpfl1="/tmp/db_spaces1"
  tmpfl2="/tmp/db_spaces2"
  tmpflt="/tmp/db_spacest"

  if (length(level) == 0) level = 1
  #
  # Check disk usage
  #
  stat_d()

  # Cleanup temp files
  system("rm /tmp/db_spaces* > /dev/null 2>&1")

  exit
}

